@@ -1,5 +1,5 @@
 # typedload
-# Copyright (C) 2019 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2019-2021 Salvo "LtWorf" Tomaselli
 #
 # typedload is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,11 +38,15 @@ class TestTypeddictLoad(unittest.TestCase):
         o = {'name': 'pino', 'age': 1.1}
         assert load(o, Person) == o
         assert load({'val': 3}, A) == {'val': '3'}
+        assert load({'val': 3, 'vil': 4}, A) == {'val': '3'}
 
         with self.assertRaises(ValueError):
             o.pop('age')
             load(o, Person)
 
+        with self.assertRaises(ValueError):
+            load({'val': 3, 'vil': 4}, A, failonextra=True)
+
     def test_is_typeddict(self):
         assert typechecks.is_typeddict(A)
         assert typechecks.is_typeddict(Person)
