@@ -28,6 +28,7 @@ from typing import *
 
 from .exceptions import *
 from .typechecks import *
+from .helpers import tname
 
 
 __all__ = [
@@ -134,6 +135,8 @@ class Loader:
         There is an internal cache to speed up lookup, so after the
         first call to load, this should no longer be modified.
 
+    strconstructed: Set of types to construct from a string.
+
     frefs: Dictionary to resolve ForwardRef.
         Something like
         class Node(NamedTuple):
@@ -190,6 +193,17 @@ class Loader:
         # Which key is used in metadata to perform name mangling
         self.mangle_key = 'name'
 
+        # Objects loaded from a string
+        self.strconstructed = {
+            Path,
+            ipaddress.IPv4Address,
+            ipaddress.IPv6Address,
+            ipaddress.IPv4Network,
+            ipaddress.IPv6Network,
+            ipaddress.IPv4Interface,
+            ipaddress.IPv6Interface
+        }
+
         # The list of handlers to use to load the data.
         # It gets iterated in order, and the first condition
         # that matches is used to load the value.
@@ -209,11 +223,9 @@ class Loader:
             (is_literal, _literalload),
             (is_typeddict, _namedtupleload),
             (lambda type_: type_ in {datetime.date, datetime.time, datetime.datetime}, _datetimeload),
-            (lambda type_: type_ == Path, _pathload),
-            (lambda type_: type_ in {ipaddress.IPv4Address, ipaddress.IPv6Address,
-                                     ipaddress.IPv4Network, ipaddress.IPv6Network,
-                                     ipaddress.IPv4Interface, ipaddress.IPv6Interface}, _ipaddressload),
+            (lambda type_: type_ in self.strconstructed, _strconstructload),
             (is_attrs, _attrload),
+            (is_any, _anyload),
         ]  # type: List[Tuple[Callable[[Any], bool], Callable[[Loader, Any, Type], Any]]]
 
         for k, v in kwargs.items():
@@ -256,16 +268,16 @@ class Loader:
                 self._indexcache[type_] = index
             except ValueError:
                 raise TypedloadTypeError(
-                    'Cannot deal with value of type %s' % type_,
+                    'Cannot deal with value of type %s' % tname(type_),
                     value=value,
                     type_=type_
                 )
 
         # Add type to known types, to resolve ForwardRef later on
         if self.frefs is not None and hasattr(type_, '__name__'):
-            tname = type_.__name__
-            if tname not in self.frefs:
-                self.frefs[tname] = type_
+            typename = type_.__name__
+            if typename not in self.frefs:
+                self.frefs[typename] = type_
 
         func = self.handlers[index][1]
 
@@ -302,6 +314,10 @@ def _forwardrefload(l: Loader, value: Any, type_: type) -> Any:
     return l.load(value, t, annotation=Annotation(AnnotationType.FORWARDREF, tname))
 
 
+def _anyload(l: Loader, value: Any, type_: type) -> Any:
+    return value
+
+
 def _literalload(l: Loader, value: Any, type_: type) -> Any:
     """
     Checks if the value is within the allowed literals and
@@ -309,7 +325,7 @@ def _literalload(l: Loader, value: Any, type_: type) -> Any:
     """
     if value in literalvalues(type_):
         return value
-    raise TypedloadValueError('Not one of the allowed values in %s' % type_, value=value, type_=type_)
+    raise TypedloadValueError('Not one of the allowed values in %s' % tname(type_), value=value, type_=type_)
 
 
 def _basicload(l: Loader, value: Any, type_: type) -> Any:
@@ -333,7 +349,7 @@ def _basicload(l: Loader, value: Any, type_: type) -> Any:
             except Exception as e:
                 raise TypedloadException(str(e), value=value, type_=type_)
         else:
-            raise TypedloadValueError('Not of type %s' % type_, value=value, type_=type_)
+            raise TypedloadValueError('Got %s of type %s, expected %s' % (repr(value), tname(type(value)), tname(type_)), value=value, type_=type_)
     return value
 
 
@@ -381,6 +397,8 @@ def _frozensetload(l: Loader, value, type_) -> FrozenSet:
     """
     This loads into something like FrozenSet[int]
     """
+    if isinstance(value, dict):
+        raise TypedloadTypeError('Unable to load dictionary as a frozenset', value=value, type_=type_)
     t = type_.__args__[0]
     return frozenset(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, v in enumerate(value))
 
@@ -400,9 +418,9 @@ def _tupleload(l: Loader, value, type_) -> Tuple:
         return tuple(l.load(v, args[0], annotation=Annotation(AnnotationType.INDEX, i)) for i, v in enumerate(value))
     else: # Tuple[something, something, somethingelse]
         if l.failonextra and len(value) > len(args):
-            raise TypedloadValueError('Value is too long for type %s' % type_, value=value, type_=type_)
+            raise TypedloadValueError('Value is too long for type %s' % tname(type_), value=value, type_=type_)
         elif len(value) < len(args):
-            raise TypedloadValueError('Value is too short for type %s' % type_, value=value, type_=type_)
+            raise TypedloadValueError('Value is too short for type %s' % tname(type_), value=value, type_=type_)
         return tuple(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, (v, t) in enumerate(zip(value, args)))
 
 
@@ -418,7 +436,12 @@ def _mangle_names(namesmap: Dict[str, str], value: Dict[str, Any]) -> Dict[str,
         return value
     r = {}
 
+    # Disallow the python names if they are in the map
+    skip = set(namesmap.values())
+
     for k, v in value.items():
+        if k in skip and k not in namesmap:
+            continue
         if k in namesmap:
             k = namesmap[k]
         r[k] = v
@@ -463,7 +486,7 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
         raise TypedloadValueError(
             'Value does not contain fields: %s which are necessary for type %s' % (
                 necessary_fields.difference(vfields),
-                type_
+                tname(type_)
             ),
             value=value,
             type_=type_,
@@ -473,7 +496,7 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
     if l.failonextra and len(fieldsdiff):
         extra = ', '.join(fieldsdiff)
         raise TypedloadValueError(
-            'Dictionary has unrecognized fields: %s and cannot be loaded into %s' % (extra, type_),
+            'Dictionary has unrecognized fields: %s and cannot be loaded into %s' % (extra, tname(type_)),
             value=value,
             type_=type_,
         )
@@ -528,7 +551,7 @@ def _unionload(l: Loader, value, type_) -> Any:
         except Exception as e:
             exceptions.append(e)
     raise TypedloadValueError(
-        'Value could not be loaded into %s' % type_,
+        'Value of %s could not be loaded into %s' % (tname(value_type), tname(type_)),
             value=value,
             type_=type_,
             exceptions=exceptions
@@ -561,7 +584,7 @@ def _enumload(l: Loader, value, type_) -> Enum:
         except Exception as e:
             exceptions.append(e)
     raise TypedloadValueError(
-        'Value could not be loaded into %s' % type_,
+        'Value of %s could not be loaded into %s' % (tname(type(value)), tname(type_)),
         value=value,
         type_=type_,
         exceptions=exceptions
@@ -587,7 +610,7 @@ def _datetimeload(l: Loader, value, type_) -> Union[datetime.date, datetime.time
 
 def _attrload(l, value, type_):
     if not isinstance(value, dict):
-        raise TypedloadTypeError('Expected dictionary, got %s' % type(value), type_=type_, value=value)
+        raise TypedloadTypeError('Expected dictionary, got %s' % tname(type(value)), type_=type_, value=value)
     value = value.copy()
     names = []
     defaults = {}
@@ -616,14 +639,7 @@ def _attrload(l, value, type_):
     return _namedtupleload(l, value, t)
 
 
-def _pathload(l: Loader, value, type_) -> Path:
-    try:
-        return Path(value)
-    except TypeError as e:
-        raise TypedloadTypeError(str(e), type_=type_, value=value)
-
-
-def _ipaddressload(l: Loader, value, type_):
+def _strconstructload(l: Loader, value, type_):
     """
     Loader for all the types in the stdlib ipaddress module.
     """
@@ -631,3 +647,7 @@ def _ipaddressload(l: Loader, value, type_):
         return type_(value)
     except ValueError as e:
         raise TypedloadValueError(str(e), type_=type_, value=value)
+    except TypeError as e:
+        raise TypedloadTypeError(str(e), type_=type_, value=value)
+    except Exception as e:
+        raise TypedloadException(str(e), type_=type, value=value)
